{############################################################### KIM1KIM.INC }
{ Purpose...: KIM1 include file: Send/receive KIM files
{ DependsOn.: LIB.INC, LIBSIO.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 22.01.2023
{############################################################################}

{*************************************************************** definitions }
CONST m_iKIM_BAUD_RATE:  INTEGER  = 600;         { baudrate for .KIM files   }
CONST m_iKIM_TXDLY_CHAR: INTEGER  = 0;           { tx delay between chars    }
CONST m_iKIM_TXDLY_LINE: INTEGER  = 5000;        { tx delay detween lines    }

{************************************************************ implementation }
{============================================================= Kim_InitSio() }
{ Purpose...: Init/deinit SIO
{ Input.....: i_fInit: TRUE= init, FALSE= de-init
{============================================================================}
PROCEDURE Kim_InitSio(i_fInit: BOOLEAN);

BEGIN

  LibSio_Init(i_fInit, m_iKIM_BAUD_RATE, 'N', 8, 2,
              m_iKIM_TXDLY_CHAR, m_iKIM_TXDLY_LINE);
END;
{========================================================= eoF Kim_InitSio() }

{============================================================ Kim_SendFile() }
{ Purpose...: Send .KIM file to SIO
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Kim_SendFile: BYTE;

  VAR bSio       : BYTE;
  VAR pstLine    : g_tpStLine;
  VAR iBufLine   : INTEGER;
  VAR fLineValid : BOOLEAN;
  VAR cKbd       : CHAR;
  VAR fStart     : BOOLEAN;

BEGIN
  bSio := g_bLibSioOK;

  {--------------------------------------------------- wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_TRANSMIT, g_stFile, ' (', File_GetBufLines, ' lines)');
  WRITE('Continue (Y/N)? ');
  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSio := g_bLibSioCANCEL;
  END;

  {---------------------------------------------------------------- init SIO }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    Kim_InitSio(TRUE);
    LibSio_RxClear;
    bSio := Term_WakeUp(FALSE);
  END;

  pstLine := PTR(1);
  fStart := TRUE;
  iBufLine := 0;

  {--------------------------------------------------- send the load command }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    bSio := LibSio_TxWait('L', '', 1000, FALSE);
  END;

  WRITELN;

  IF (bSio = g_bLibSioOK) THEN BEGIN
    WRITELN(g_stMSG_TRANSMITTING, g_stFile, g_stMSG_CANCEL);
  END;

  {----------------------------------------------------------- send the file }
  WHILE ((pstLine <> NIL) AND (bSio = g_bLibSioOK)) DO BEGIN
    pstLine := File_GetString(iBufLine, fStart, NIL);
    fStart := FALSE;
    iBufLine := -1;

    {--------------------------------------------- send and display one line }
    IF (pstLine <> NIL) THEN BEGIN

      {------------------------------------ write the line to screen and SIO }
      bSio := LibSio_TxString(pstLine^ + g_cCR, TRUE);
      WRITELN;
    END;
  END;

  WRITELN;

  IF (bSio = g_bLibSioCANCEL) THEN BEGIN
    WRITE('Cancelled. ');
  END;

  Kim_InitSio(FALSE);
  Kim_SendFile := bSio;
END;
{======================================================== eoP Kim_SendFile() }

{========================================================= Kim_ReceiveFile() }
{ Purpose...: Receive KIM file
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Kim_ReceiveFile: BYTE;

  CONST iEND_TIMEOUT : INTEGER = 1500;

  VAR fCont        : BOOLEAN;
  VAR bSio         : BYTE;
  VAR cKbd         : CHAR;
  VAR cRx          : CHAR;
  VAR bRx          : BYTE;
  VAR pbRx         : ^BYTE;
  VAR fRx          : BOOLEAN;
  VAR iTimeout     : INTEGER;
  VAR fWriteBuffer : BOOLEAN;
  VAR iFromAddr    : INTEGER;
  VAR iToAddr      : INTEGER;
  VAR piFromAddr   : ^INTEGER;
  VAR piToAddr     : ^INTEGER;

BEGIN
  bSio := g_bLibSioCANCEL;
  iTimeout := -1;
  pbRx := PTR(ADDR(cRx));
  fRx := FALSE;
  fWriteBuffer := FALSE;

  CLRSCR;
  WRITELN(g_stMSG_RECEIVE, g_stFile, ' (', File_GetBufSize, ' bytes free)');

  {-------------------------------- get from and to addresses from keyboard }
  piFromAddr := PTR(ADDR(iFromAddr));
  piToAddr := PTR(ADDR(iToAddr));
  fCont := Lib_GetHex('From', piFromAddr, 4, 'To', piToAddr, 4, '', NIL, 0);

  {---------------------------------------------- init SIO and wake up KIM1 }
  IF (fCont = TRUE) THEN BEGIN
    Kim_InitSio(TRUE);
    LibSio_RxClear;
    WRITELN;
    bSio := Term_WakeUp(FALSE);

    {------------------------------------------------------ send to address }
    IF (bSio = g_bLibSioOK) THEN BEGIN
      WRITELN;
      WRITE('Preparing');
      bSio := Term_SendData($17F7, iToAddr, 2, FALSE);
    END;

    WRITE('.');

    {---------------------------------------- set address to 'from' address }
    IF (bSio = g_bLibSioOK) THEN BEGIN
      bSio := Term_SendData(iFromAddr, 0, 0, FALSE);
    END;

    WRITE('.');

    {------------------------------------------------ send the list command }
    IF (bSio = g_bLibSioOK) THEN BEGIN
      bSio := LibSio_TxWait('Q', '', 1000, FALSE);
    END;

    WRITE(g_cCR);
    CLREOL;

    IF (bSio = g_bLibSioOK) THEN BEGIN
      WRITELN(g_stMSG_RECEIVING, g_stFile, g_stMSG_CANCEL);
      bSio := g_bLibSioNOK;
      LibSio_RxClear;
    END;
  END;

  {-------------------------------------------- loop until done or cancelled }
  WHILE (bSio = g_bLibSioNOK) DO BEGIN
    bSio := LibSio_RxByte(pbRx, iTimeout, TRUE);
    cRx := CHR(ORD(cRx));

    {-------------------------------------------- one character was received }
    IF (bSio = g_bLibSioOK) THEN BEGIN
      bSio := g_bLibSioNOK;
      iTimeout := iEND_TIMEOUT;
      bRx := ORD(cRx);
      fRx := TRUE;

      {-------------------------------------- write character to file buffer }
      IF (File_PutBuffer(cRx) = FALSE) THEN BEGIN
        bSio := g_bLibSioCANCEL;
      END;

      {------------------------------------------- write character to screen }
      IF ((bSio = g_bLibSioNOK) AND (bRx <> 0)) THEN BEGIN
        WRITE(cRx);
      END;
    END;
  END;

  {----------------------------------------------------- save buffer to disk }
  IF (bSio = g_bLibSioTIMEOUT) THEN BEGIN
    fWriteBuffer := fRx;
    bSio := g_bLibSioOK;
  END;

  {--------------------------------------------------------------- cancelled }
  IF ((bSio = g_bLibSioCANCEL) AND (fRx = TRUE)) THEN BEGIN
    WRITELN;
    WRITE('Cancelled. Write buffer to ', g_stFile, ' ? (Y/N) ');
    cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
    fWriteBuffer := (cKbd = 'Y');
  END;

  WRITELN;

  IF (fWriteBuffer = TRUE) THEN BEGIN
    File_WriteBuffer;
    WRITELN;
  END;

  Kim_InitSio(FALSE);
  Kim_ReceiveFile := bSio;
END;
{===================================================== eoF Kim_ReceiveFile() }

{########################################################### EOF KIM1KIM.INC }
