{############################################################## KIM1TERM.INC }
{ Purpose...: KIM1 terminal
{ DependsOn.: LIB.INC, LIBSIO.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 22.01.2023
{############################################################################}

{*************************************************************** definitions }
CONST m_iTERM_BAUD_RATE:  INTEGER  = 600;        { baudrate                  }
CONST m_iTERM_TXDLY_CHAR: INTEGER  = 0;          { tx delay between chars    }
CONST m_iTERM_TXDLY_LINE: INTEGER  = 0;          { tx delay between lines    }

{************************************************************ implementation }
{============================================================ Term_InitSio() }
{ Purpose...: Init/deinit SIO
{ Input.....: i_fInit: TRUE= init, FALSE= de-init
{============================================================================}
PROCEDURE Term_InitSio(i_fInit: BOOLEAN);

BEGIN

  LibSio_Init(i_fInit, m_iTERM_BAUD_RATE, 'N', 8, 2,
              m_iTERM_TXDLY_CHAR,m_iTERM_TXDLY_LINE);
END;
{======================================================== eoF Term_InitSio() }

{============================================================ Term_Message() }
{ Purpose...: Print a message to screen
{ Input.....: i_stMsg.: Base message
{             i_fOnOff: TRUE= print ON or OFF
{             i_fStat.: TRUE= ON, FALSE= OFF
{============================================================================}
PROCEDURE Term_Message(i_stMsg: g_tStLine;
                       i_fOnOff: BOOLEAN; i_fStat: BOOLEAN);

BEGIN
  WRITELN;
  WRITE('-=[', i_stMsg);

  IF (i_fOnOff = TRUE) THEN BEGIN
    WRITE(' is ');

    IF (i_fStat) = TRUE THEN BEGIN
      WRITE('ON');
    END ELSE BEGIN
      WRITE('OFF');
    END;
  END;

  WRITELN(']=-');
END;
{======================================================== eoP Term_Message() }

{============================================================= Term_Wakeup() }
{ Purpose...: Try to connect to KIM1 by sending a DEL, if alive, KIM1
{             sends back a 'KIM'<CR/LF><ADDRESS> <DATA> sequence
{ Input.....: i_fWrite: TRUE= show dialog, FALSE= be silent
{ Result....: One of g_bLibSio<result>
{=========================================================================== }
FUNCTION Term_WakeUp(i_fWrite: BOOLEAN): BYTE;

  VAR bSio  : BYTE;
  VAR fHint : BOOLEAN;
  VAR fDone : BOOLEAN;

BEGIN
  fHint := FALSE;
  fDone := FALSE;

  WHILE (fDone = FALSE) DO BEGIN
    bSio := LibSio_TxByte($7F);

    bSio := LibSio_TxWait('',
                          CHR($7F) + g_stCRLF + 'KIM' + g_stCRLF + 'xxxx xx ',
                          1000, i_fWrite);

    IF (bSio <> g_bLibSioOK) THEN BEGIN
      IF ((bSio <> g_bLibSioCANCEL) AND (fHint = FALSE)) THEN BEGIN
        fHint := TRUE;
        WRITE('Waking up KIM1 ... ');
        WRITE('(might need to switch to TTY and hit RS key) ');
      END ELSE BEGIN
        fDone := (bSio = g_bLibSioCANCEL);
      END;
    END;

    IF (fDone = FALSE) THEN BEGIN
      fDone := (bSio = g_bLibSioOK);
    END;

    IF (fDone = TRUE) THEN BEGIN
      IF ((fHint = TRUE) AND (i_fWrite = FALSE)) THEN BEGIN
        WRITE(g_cCR);
        CLREOL;
      END;
    END;
  END;

  Term_WakeUp := bSio;
END;
{========================================================= eoF Term_WakeUp() }

{=========================================================== Term_SendData() }
{ Purpose...: Send data to secified address
{ Input.....: i_iAddress: Address where to store data
{             i_iData...: Data to store
{             i_bLen....: Number of bytes to store, 0-2, 0=set address only
{             i_fWrite..: TRUE= write to screen, FALSE= be silent
{ Result....: One of g_bLibSioXXX
{============================================================================}
FUNCTION Term_SendData(i_iAddress: INTEGER; i_iData: INTEGER; i_bLen: BYTE;
                       i_fWrite: BOOLEAN): BYTE;

  VAR bSio  : BYTE;
  VAR stHex : g_tstHexAddr;

BEGIN
  {-------------------------------------------------------------- set address }
  stHex := Lib_CnvBinToHex(i_iAddress, 4);
  bSio := LibSio_TxWait(stHex + ' ', g_stCRLF + stHex + ' xx ', -1, i_fWrite);

  {------------------------------------------------------- send data low byte }
  IF ((bSio = g_bLibSioOK) AND (i_bLen <> 0)) THEN BEGIN
    stHex := Lib_CnvBinToHex((i_iData AND $00FF), 2);
    bSio := LibSio_TxWait(stHex + '.', g_stCRLF + 'xxxx xx ', -1, i_fWrite);
  END;

  {-------------------------------------------------------- send data hi byte }
  IF ((bSio = g_bLibSioOK) AND (i_bLen = 2)) THEN BEGIN
    stHex := Lib_CnvBinToHex((i_iData SHR 8), 2);
    bSio := LibSio_TxWait(stHex + '.', g_stCRLF + 'xxxx xx ', -1, i_fWrite);
  END;

  Term_SendData := bSio;
END;
{======================================================= eoF Term_SendData() }

{============================================================ Term_Vectors() }
{ Purpose...: Setup NMI and IRQ vectors to default (debug) addresses
{ Input.....: i_fWrite: TRUE= write dialog to screen, FALSE= be silent
{ Result....: One of g_bLibSio<result>
{=========================================================================== }
FUNCTION Term_Vectors(i_fWrite : BOOLEAN): BYTE;

  VAR bSio  : BYTE;

BEGIN
  WRITELN;
  WRITELN('Set NMI and IRQ vectors to 1C00');
  bSio := Term_WakeUp(FALSE);

  IF (bSio = g_bLibSioOK) THEN BEGIN

    {----------------------------------------- send the NMI vector address }
    bSio := Term_SendData($17FA, $1C00, 2, i_fWrite);

    {----------------------------------------- send the IRQ vector address }
    IF (bSio = g_bLibSioOK) THEN BEGIN
      bSio := Term_SendData($17FE, $1C00, 2, i_fWrite);
    END;
  END;

  Term_Vectors := bSio;
END;
{======================================================== eoF Term_Vectors() }

{=========================================================== Term_TapeSave() }
{ Purpose...: Get from and to addresses and file ID from console, store
{ Input.....: i_fWrite: TRUE= write dialog to screen, FALSE= be silent
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Term_TapeSave(i_fWrite : BOOLEAN): BYTE;

  VAR bSio   : BYTE;
  VAR iFrom  : INTEGER;
  VAR iTo    : INTEGER;
  VAR iId    : INTEGER;
  VAR piFrom : ^INTEGER;
  VAR piTo   : ^INTEGER;
  VAR piId   : ^INTEGER;
  VAR fCont  : BOOLEAN;

BEGIN
  bSio := g_bLibSioNOK;
  piFrom := PTR(ADDR(iFrom));
  piTo := PTR(ADDR(iTo));
  piId := PTR(ADDR(iId));

  WRITELN;
  WRITE('Write to tape  ');
  fCont := Lib_GetHex('From', piFrom, 4, 'To', piTo, 4, 'ID', piId, 2);

  IF (fCont = TRUE) THEN BEGIN
    bSio := Term_WakeUp(FALSE);
  END;

  {---------------------------------------------------- clear decimal mode }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    WRITELN;
    bSio := Term_SendData($00F1, $0000, 1, i_fWrite);
  END;

  {------------------------------------------------- send the from address }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    bSio := Term_SendData($17F5, iFrom, 2, i_fWrite);
  END;

  {--------------------------------------------------- send the to address }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    iTo := iTo + 1;
    bSio := Term_SendData($17F7, iTo, 2, i_fWrite);
  END;

  {----------------------------------------------------------- send the ID }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    bSio := Term_SendData($17F9, iId, 1, i_fWrite);
  END;

  {-------------------------------------------------- set the exec address }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    bSio := Term_SendData($1800, 0, 0, i_fWrite);
  END;

  IF (bSio = g_bLibSioOK) THEN BEGIN
    WRITELN;
    WRITELN('Start record, then press `G`');
  END ELSE BEGIN
    WRITE(g_cCR);
    CLREOL;
  END;

  Term_Vectors := bSio;
END;
{======================================================= eoF Term_TapeSave() }

{=========================================================== Term_TapeLoad() }
{ Purpose...: Get file ID from console, store
{ Input.....: i_fWrite: TRUE= write dialog to screen, FALSE= be silent
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Term_TapeLoad(i_fWrite : BOOLEAN): BYTE;

  VAR bSio  : BYTE;
  VAR iId   : INTEGER;
  VAR piId  : ^INTEGER;
  VAR fCont : BOOLEAN;

BEGIN
  bSio := g_bLibSioNOK;
  piId := PTR(ADDR(iId));

  WRITELN;
  WRITE('Load from tape  ');
  fCont := Lib_GetHex('ID', piId, 2, '', NIL, 0, '', NIL, 0);

  IF (fCont = TRUE) THEN BEGIN
    bSio := Term_WakeUp(FALSE);
  END;

  {---------------------------------------------------- clear decimal mode }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    WRITELN;
    bSio := Term_SendData($00F1, $0000, 1, i_fWrite);
  END;

  {----------------------------------------------------------- send the ID }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    bSio := Term_SendData($17F9, iId, 1, i_fWrite);
  END;

  {-------------------------------------------------- set the exec address }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    bSio := Term_SendData($1873, 0, 0, i_fWrite);
  END;

  IF (bSio = g_bLibSioOK) THEN BEGIN
    WRITELN;
    WRITELN('Start play, then press `G`');
  END ELSE BEGIN
    WRITE(g_cCR);
    CLREOL;
  END;

  Term_Vectors := bSio;
END;
{======================================================= eoF Term_TapeLoad() }

{==================================================================== Term() }
{ Purpose...: A thumb terminal
{============================================================================}
PROCEDURE Term;

  CONST HEX_CHARS_PER_LINE  = 16;

  VAR fExit    : BOOLEAN;
  VAR fEcho    : BOOLEAN;
  VAR fPrinter : BOOLEAN;
  VAR fSkip    : BOOLEAN;
  VAR fPrint   : BOOLEAN;
  VAR fRx      : BOOLEAN;
  VAR fHex     : BOOLEAN;
  VAR bHexCnt  : BYTE;
  VAR bRx      : BYTE;
  VAR pbRx     : g_tpByte;
  VAR bHex     : BYTE;
  VAR cKbd     : CHAR;
  VAR stRx     : g_tstLine;
  VAR stTx     : g_tstLine;
  VAR bDummy   : BYTE;

BEGIN
  fExit := FALSE;
  fEcho := FALSE;
  fSkip := FALSE;
  fPrinter := FALSE;
  fRx := FALSE;
  fHex := FALSE;
  bHexCnt := 0;
  pbRx := PTR(ADDR(bRx));
  stRx := '';
  stTx := '';

  CLRSCR;
  WRITELN('Commands: <^E>cho | <^P>rint | HE<^X> | <^C>lear', g_stMSG_CANCEL);
  WRITELN('Macros  : <^V>ectors | <^S>ave | <^L>oad');

  Term_InitSio(TRUE);
  fExit := (Term_WakeUp(TRUE) <> g_bLibSioOK);

  {---------------------------------------------------- loop until cancelled }
  WHILE (fExit = FALSE) DO BEGIN

    {-------------------------------------------- display received character }
    IF (LibSio_RxByte(pbRx, 0, FALSE) = g_bLibSioOK) THEN BEGIN
      bHex := bRx;
      bRx := (bRx AND $7F);
      stRx := '';

      {------------------------------------------------- write data as ASCII }
      IF (fHex = FALSE) THEN BEGIN

        IF (bRx <> 0) THEN BEGIN
          stRx := CONCAT(stRx, CHR(bRx));
        END;
      END

      {---------------------------------------- write data as HEX characters }
      ELSE BEGIN

        IF (fRx = FALSE) THEN BEGIN
          fRx := TRUE;
          bHexCnt := 0;
          stRx := CONCAT(stRx, g_stCRLF);
        END;

        stRx := CONCAT(stRx, '[', Lib_CnvBinToHex(bHex, 2), ']');
        bHexCnt := bHexCnt + 1;

        {----------------------------------------------- format display line }
        IF (bHexCnt = (HEX_CHARS_PER_LINE / 2)) THEN BEGIN
          stRx := CONCAT(stRx, '  ');

        END ELSE BEGIN

          IF ((bHexCnt <> 0) AND (bHexCnt MOD 4 = 0)) THEN BEGIN
            stRx := CONCAT(stRx, ' ');
          END;
        END;

        IF (bHexCnt = HEX_CHARS_PER_LINE) THEN BEGIN
          bHexCnt := 0;
          stRx := CONCAT(stRx, g_stCRLF);
        END;
      END;
    END;

    {----------------------------------------- send character from keyboard }
    IF (KEYPRESSED) THEN BEGIN
      READ(KBD, cKbd);
      cKbd := UPCASE(cKbd);
      stTx := '';
      fSkip := FALSE;

      {--------------------------------------------------- clear the screen }
      IF (cKbd = g_cCTRLC) THEN BEGIN
        CLRSCR;
        fSkip := TRUE;
      END;

      {-------------------------------------------- toggle HEX data display }
      IF (cKbd = g_cCTRLX) THEN BEGIN
        fHex := (NOT fHex);
        Term_Message('Hex mode', TRUE, fHex);
        bHexCnt := 0;
        fSkip := TRUE;
      END;

      {------------------------------------------- toggle local screen echo }
      IF (cKbd = g_cCTRLE) THEN BEGIN
        fEcho := (NOT fEcho);
        Term_Message('Echo', TRUE, fEcho);
        fSkip := TRUE;
      END;

      {------------------------------------------------- toggle printer echo }
      IF (cKbd = g_cCTRLP) THEN BEGIN
        fPrinter := NOT fPrinter;

        IF ((LibPrt_Ready = FALSE) AND (fPrinter = TRUE)) THEN BEGIN
          Term_Message('Printer not ready', FALSE, FALSE);
          fPrinter := FALSE;
        END ELSE BEGIN
          Term_Message('Printer', TRUE, fPrinter);
          LibPrt_Setup(0, 0, TRUE, -1);
        END;

        fSkip := TRUE;
      END;

      {----------------------------------------------- set interrupt vectors }
      IF (cKbd = g_cCTRLV) THEN BEGIN
        bDummy := Term_Vectors(TRUE);
        fSkip := TRUE;
      END;

      {----------------------------------------------- prepare for tape save }
      IF (cKbd = g_cCTRLS) THEN BEGIN
        bDummy := Term_TapeSave(TRUE);
        fSkip := TRUE;
      END;

      {----------------------------------------------- prepare for tape load }
      IF (cKbd = g_cCTRLL) THEN BEGIN
        bDummy := Term_TapeLoad(TRUE);
        fSkip := TRUE;
      END;

      {--------------------------------------------------- <esc> was pressed }
      IF (cKbd = g_cESC) THEN BEGIN
        WRITELN;
        fExit := TRUE;
        fSkip := TRUE;
      END;

      {---------------------------- process keyboard char if no function key }
      IF (fSkip = FALSE) THEN BEGIN

        {------------------------------------------------------- normal mode }
        IF ((fEcho = TRUE) AND (fHex = FALSE)) THEN BEGIN
          stTx := CONCAT(stTx, cKbd);
          IF (cKbd = g_cCR) THEN stTx := CONCAT(stTx, g_cLF);
        END;

        {---------------------------------------------------------- HEX mode }
        IF ((fEcho = TRUE) AND (fHex = TRUE)) THEN BEGIN
          IF (fRx = TRUE) THEN BEGIN
            fRx := FALSE;
            bHexCnt := 0;
            stTx := CONCAT(stTx, g_stCRLF);
          END;

          stTx := CONCAT(stTx, '{', Lib_CnvBinToHex(ORD(cKbd), 2), '}');
          bHexCnt := bHexCnt + 1;

          IF (bHexCnt = HEX_CHARS_PER_LINE) THEN BEGIN
            bHexCnt := 0;
            stTx := CONCAT(stTx, g_stCRLF);
          END;
        END;

        fExit := (LibSio_TxByte(ORD(cKbd)) <> g_bLibSioOK);
      END;
    END;

    {-------------------------------------------------------------- printing }
    fPrint := FALSE;

    IF (LENGTH(stRx) <> 0) THEN BEGIN
      WRITE(stRx);
      fPrint := TRUE;
    END;

    IF (LENGTH(stTx) <> 0) THEN BEGIN
      WRITE(stTx);
      fPrint := TRUE;
    END;

    IF ((fPrinter = TRUE) AND (fPrint = TRUE)) THEN BEGIN
      LibSio_SetTxHs(g_bLibSioTX_HS_OFF);

      IF (LENGTH(stRx) <> 0) THEN BEGIN
        fPrinter := LibPrt_Print(CHR(0), stRx);
      END;

      IF (LENGTH(stTx) <> 0) THEN BEGIN
        fPrinter := LibPrt_Print(CHR(0), stTx);
      END;

      WHILE ((fPrinter = TRUE) AND (LibPrt_Ready = FALSE)) DO BEGIN END;
      LibSio_SetTxHs(g_bLibSioTX_HS_ON);
    END;

    stRx := '';
    stTx := '';
  END;

  Term_InitSio(FALSE);
END;
{================================================================ eoP Term() }

{########################################################## EOF KIM1TERM.INC }
